
/* 
 * Copyright (c) 1993-2003,2012 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/*------------------  Fichier gnr automatiquement ------------------*/
/*------------------  Ne pas diter manuellement !!! ------------------*/

#ifndef imu_MSG_LIB_H
#define imu_MSG_LIB_H

#include "portLib.h"
#include "csLib.h"
#include "posterLib.h"
 
#include "imuType.h"

#define TIME_WAIT_REPLY           /*2000*/ 0
 



/*----------- Tailles des messages et boites aux lettres  ----------*/

/* Taille du buffer utilise dans la bal pour un mesage de taille size
 * (cf servMbox pour UNIX et h2rgnLib pour VXWORKS )
 */


#define BUF_SIZE(size) \
   (size + 4 - (size & 3) + sizeof(LETTER_HDR) + 8)

/**
 ** Boite aux lettres reception requetes 
 **/
#define  IMU_MBOX_NAME                  genomInstanceName("imu")

#define  IMU_MAX_RQST_SIZE              88

#define  IMU_MBOX_RQST_SIZE       \
 (BUF_SIZE(IMU_MAX_RQST_SIZE) * SERV_NMAX_RQST_ID)

/** 
 ** Taille max repliques (serveur)
 **/
#define  IMU_MAX_INTERMED_REPLY_SIZE   (sizeof(int))

#define  IMU_MAX_REPLY_SIZE             88

/** 
 ** Taille bal clients pour reception repliques 
 **/
#define  IMU_CLIENT_MBOX_REPLY_SIZE   \
((BUF_SIZE(IMU_MAX_REPLY_SIZE) \
   + BUF_SIZE(IMU_MAX_INTERMED_REPLY_SIZE)) \
 * CLIENT_NMAX_RQST_ID)

/*-------------------------- Liste des requetes  -------------------------*/

#define IMU_INIT_RQST 0
#define IMU_GETINITPARAMS_RQST 1
#define IMU_ZERO_RQST 2
#define IMU_GETDEVICEVERSION_RQST 3
#define IMU_SETACCELBIASES_RQST 4
#define IMU_COMPUTEACCELBIASES_RQST 5
#define IMU_COMPUTEANGSPEEDSBIASES_RQST 6
#define IMU_DUMPIMUDATAONFILE_RQST 7
#define IMU_DUMPIMUDATASTOP_RQST 8
#define IMU_GETSENSORSDATA_RQST 9
#define IMU_SETSENSORSDATANUMFILE_RQST 10
#define IMU_INITINTEGRATEDDATA_RQST 11
#define IMU_SETVARIANCES_RQST 12
#define IMU_FILTERIMUDATA_RQST 13
#define IMU_GETFILTERPARAMS_RQST 14
#define IMU_BACKSTEPATTITUDEESTIMATION_RQST 15
#define IMU_STOPBACKSTEPATTITUDEESTIMATION_RQST 16
#define IMU_GETBACKSTEPATTITUDEESTIMATIONPARAMS_RQST 17
#define IMU_SETHEADINGPOSTER_RQST 18
#define IMU_FLATTERRAINMONITOR_RQST 19
#define IMU_STOPMONITOR_RQST 20
#define IMU_GETMONITORPARAMS_RQST 21
#define IMU_ABORT_RQST 22


/*---------------- PROTOTYPES DES FONCTIONS EXTERNES ------------------*/

#ifdef __cplusplus
extern "C" {
#endif

STATUS imuClientInit (CLIENT_ID *pClientId);

STATUS imuClientEnd (CLIENT_ID clientId);

int imuAbortRqstSend (CLIENT_ID clientId, 
			   int *pRqstId,
			   int *activity,
			   int replyTimeOut);

int imuAbortReplyRcv (CLIENT_ID clientId, 
			   int rqstId, 
			   int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			   int *bilan);

int imuAbortRqstAndRcv (CLIENT_ID clientId, 
			     int *activity,
			     int *bilan);




extern STATUS imuInitRqstSend (CLIENT_ID clientId, int *pRqstId,
					 ImuDevice *(in_device),
					 int replyTimeOut);

extern int imuInitReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuInitRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					ImuDevice *(in_device), 
					
					int *activity, int *bilan);

extern int imuInitRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					ImuDevice *(in_device), 
					
					int *activity,
					int *bilan);




extern STATUS imuGetInitParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int imuGetInitParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       ImuDevice *(out_device),
			       int *bilan);

int imuGetInitParamsRqstAndRcv (CLIENT_ID clientId, 
				  
				 ImuDevice *(out_device),
				 int *bilan);




extern STATUS imuZeroRqstSend (CLIENT_ID clientId, int *pRqstId,
					 double *(in_zero),
					 int replyTimeOut);

extern int imuZeroReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuZeroRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					double *(in_zero), 
					
					int *activity, int *bilan);

extern int imuZeroRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					double *(in_zero), 
					
					int *activity,
					int *bilan);




extern STATUS imuGetDeviceVersionRqstSend (CLIENT_ID clientId, int *pRqstId,
					 
					 int replyTimeOut);

extern int imuGetDeviceVersionReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuGetDeviceVersionRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					 
					
					int *activity, int *bilan);

extern int imuGetDeviceVersionRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					 
					
					int *activity,
					int *bilan);




extern STATUS imuSetAccelBiasesRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IMU_SETACCELBIASES_PARAMS *(in_setAccelBiasesParams),
					 int replyTimeOut);

int imuSetAccelBiasesReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int imuSetAccelBiasesRqstAndRcv (CLIENT_ID clientId, 
				 IMU_SETACCELBIASES_PARAMS *(in_setAccelBiasesParams), 
				 
				 int *bilan);




extern STATUS imuComputeAccelBiasesRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IMU_COMPUTEACCELBIASES_PARAMS *(in_computeAccelBiasesParams),
					 int replyTimeOut);

extern int imuComputeAccelBiasesReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuComputeAccelBiasesRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IMU_COMPUTEACCELBIASES_PARAMS *(in_computeAccelBiasesParams), 
					
					int *activity, int *bilan);

extern int imuComputeAccelBiasesRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IMU_COMPUTEACCELBIASES_PARAMS *(in_computeAccelBiasesParams), 
					
					int *activity,
					int *bilan);




extern STATUS imuComputeAngSpeedsBiasesRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IMU_COMPUTEANGSPEEDSBIASES_PARAMS *(in_computeAngSpeedsBiasesParams),
					 int replyTimeOut);

extern int imuComputeAngSpeedsBiasesReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuComputeAngSpeedsBiasesRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IMU_COMPUTEANGSPEEDSBIASES_PARAMS *(in_computeAngSpeedsBiasesParams), 
					
					int *activity, int *bilan);

extern int imuComputeAngSpeedsBiasesRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IMU_COMPUTEANGSPEEDSBIASES_PARAMS *(in_computeAngSpeedsBiasesParams), 
					
					int *activity,
					int *bilan);




extern STATUS imuDumpImuDataOnFileRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IMU_DUMPONFILE_PARAMS *(in_dumpOnFileParams),
					 int replyTimeOut);

extern int imuDumpImuDataOnFileReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuDumpImuDataOnFileRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IMU_DUMPONFILE_PARAMS *(in_dumpOnFileParams), 
					
					int *activity, int *bilan);

extern int imuDumpImuDataOnFileRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IMU_DUMPONFILE_PARAMS *(in_dumpOnFileParams), 
					
					int *activity,
					int *bilan);




extern STATUS imuDumpImuDataStopRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int imuDumpImuDataStopReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int imuDumpImuDataStopRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS imuGetSensorsDataRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IMU_GETSENSORSDATA_PARAMS *(in_getSensorsDataParams),
					 int replyTimeOut);

extern int imuGetSensorsDataReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuGetSensorsDataRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IMU_GETSENSORSDATA_PARAMS *(in_getSensorsDataParams), 
					
					int *activity, int *bilan);

extern int imuGetSensorsDataRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IMU_GETSENSORSDATA_PARAMS *(in_getSensorsDataParams), 
					
					int *activity,
					int *bilan);




extern STATUS imuSetSensorsDataNumFileRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IMU_SETSENSORSDATA_PARAMS *(in_setSensorsDataParams),
					 int replyTimeOut);

int imuSetSensorsDataNumFileReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int imuSetSensorsDataNumFileRqstAndRcv (CLIENT_ID clientId, 
				 IMU_SETSENSORSDATA_PARAMS *(in_setSensorsDataParams), 
				 
				 int *bilan);




extern STATUS imuInitIntegratedDataRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IMU_INTEGRATEDDATA_INITVALUES *(in_initValues),
					 int replyTimeOut);

extern int imuInitIntegratedDataReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuInitIntegratedDataRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IMU_INTEGRATEDDATA_INITVALUES *(in_initValues), 
					
					int *activity, int *bilan);

extern int imuInitIntegratedDataRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IMU_INTEGRATEDDATA_INITVALUES *(in_initValues), 
					
					int *activity,
					int *bilan);




extern STATUS imuSetVariancesRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IMU_VARIANCES *(in_imuVariances),
					 int replyTimeOut);

int imuSetVariancesReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int imuSetVariancesRqstAndRcv (CLIENT_ID clientId, 
				 IMU_VARIANCES *(in_imuVariances), 
				 
				 int *bilan);




extern STATUS imuFilterImuDataRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 IMU_FILTER_PARAMS *(in_imuDataFilterParams),
					 int replyTimeOut);

int imuFilterImuDataReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int imuFilterImuDataRqstAndRcv (CLIENT_ID clientId, 
				 IMU_FILTER_PARAMS *(in_imuDataFilterParams), 
				 
				 int *bilan);




extern STATUS imuGetFilterParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int imuGetFilterParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       IMU_FILTER_PARAMS *(out_imuDataFilterParams),
			       int *bilan);

int imuGetFilterParamsRqstAndRcv (CLIENT_ID clientId, 
				  
				 IMU_FILTER_PARAMS *(out_imuDataFilterParams),
				 int *bilan);




extern STATUS imuBackStepAttitudeEstimationRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IMU_BACKSTEPPING_PARAMS *(in_backStepParams),
					 int replyTimeOut);

extern int imuBackStepAttitudeEstimationReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuBackStepAttitudeEstimationRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IMU_BACKSTEPPING_PARAMS *(in_backStepParams), 
					
					int *activity, int *bilan);

extern int imuBackStepAttitudeEstimationRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IMU_BACKSTEPPING_PARAMS *(in_backStepParams), 
					
					int *activity,
					int *bilan);




extern STATUS imuStopBackStepAttitudeEstimationRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int imuStopBackStepAttitudeEstimationReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int imuStopBackStepAttitudeEstimationRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS imuGetBackStepAttitudeEstimationParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int imuGetBackStepAttitudeEstimationParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       IMU_BACKSTEPPING_PARAMS *(out_backStepParams),
			       int *bilan);

int imuGetBackStepAttitudeEstimationParamsRqstAndRcv (CLIENT_ID clientId, 
				  
				 IMU_BACKSTEPPING_PARAMS *(out_backStepParams),
				 int *bilan);




extern STATUS imuSetHeadingPosterRqstSend (CLIENT_ID clientId, int *pRqstId,
					 GEN_STRING64 *(in_headingPosterName),
					 int replyTimeOut);

extern int imuSetHeadingPosterReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuSetHeadingPosterRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					GEN_STRING64 *(in_headingPosterName), 
					
					int *activity, int *bilan);

extern int imuSetHeadingPosterRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					GEN_STRING64 *(in_headingPosterName), 
					
					int *activity,
					int *bilan);




extern STATUS imuFlatTerrainMonitorRqstSend (CLIENT_ID clientId, int *pRqstId,
					 IMU_MONITOR_PARAMS *(in_monitorParams),
					 int replyTimeOut);

extern int imuFlatTerrainMonitorReplyRcv (CLIENT_ID clientId, int rqstId, 
				      int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY 
						     BLOCK_ON_INTERMED_REPLY */
				      
				      int *activity, int *bilan);


extern int imuFlatTerrainMonitorRqstAndAck (CLIENT_ID clientId, int *pRqstId,
					int replyTimeOut,
					IMU_MONITOR_PARAMS *(in_monitorParams), 
					
					int *activity, int *bilan);

extern int imuFlatTerrainMonitorRqstAndRcv (CLIENT_ID clientId, 
					int replyTimeOut,
					IMU_MONITOR_PARAMS *(in_monitorParams), 
					
					int *activity,
					int *bilan);




extern STATUS imuStopMonitorRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int imuStopMonitorReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       
			       int *bilan);

int imuStopMonitorRqstAndRcv (CLIENT_ID clientId, 
				  
				 
				 int *bilan);




extern STATUS imuGetMonitorParamsRqstSend (CLIENT_ID clientId, 
					 int *pRqstId,
					 
					 int replyTimeOut);

int imuGetMonitorParamsReplyRcv (CLIENT_ID clientId, 
			       int rqstId, 
			       int block,  /* NO_BLOCK BLOCK_ON_FINAL_REPLY */
			       IMU_MONITOR_PARAMS *(out_monitorParams),
			       int *bilan);

int imuGetMonitorParamsRqstAndRcv (CLIENT_ID clientId, 
				  
				 IMU_MONITOR_PARAMS *(out_monitorParams),
				 int *bilan);


#ifdef __cplusplus
}
#endif
#endif /* imu_MSG_LIB_H */

